@@ -1,9 +1,4 @@
-/* Copyright 2007 Free Software Foundation
- *
- * Copying and distribution of this file, with or without modification,
- * are permitted in any medium without royalty provided the copyright
- * notice and this notice are preserved.
- */
+/* This example code is placed in the public domain. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -264,6 +259,47 @@ cert_callback (gnutls_session_t session,
   type = gnutls_certificate_type_get (session);
   if (type == GNUTLS_CRT_X509)
     {
+      /* check if the certificate we are sending is signed
+       * with an algorithm that the server accepts */
+      gnutls_sign_algorithm_t cert_algo, req_algo;
+      int i, match = 0;
+
+      ret = gnutls_x509_crt_get_signature_algorithm (crt);
+      if (ret < 0)
+	{
+	  /* error reading signature algorithm 
+	   */
+	  return -1;
+	}
+      cert_algo = ret;
+
+      i = 0;
+      do
+	{
+	  ret = gnutls_sign_algorithm_get_requested (session, i, &req_algo);
+	  if (ret >= 0 && cert_algo == req_algo)
+	    {
+	      match = 1;
+	      break;
+	    }
+
+	  /* server has not requested anything specific */
+	  if (i == 0 && ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE)
+	    {
+	      match = 1;
+	      break;
+	    }
+	  i++;
+	}
+      while (ret >= 0);
+
+      if (match == 0)
+	{
+	  printf
+	    ("- Could not find a suitable certificate to send to server\n");
+	  return -1;
+	}
+
       st->type = type;
       st->ncerts = 1;
 
