@@ -30,7 +30,7 @@ client_avp (gnutls_session_t session, void *ptr,
 
   if (last)
     printf ("- received %d bytes AVP: `%.*s'\n",
-	    lastlen, (int) lastlen, last);
+	    (int) lastlen, (int) lastlen, last);
   else
     printf ("- new application phase\n");
 
@@ -39,7 +39,7 @@ client_avp (gnutls_session_t session, void *ptr,
     return -1;
   *newlen = strlen (*new);
 
-  printf ("- sending %d bytes AVP: `%s'\n", *newlen, *new);
+  printf ("- sending %d bytes AVP: `%s'\n", (int) *newlen, *new);
 
   gnutls_ia_permute_inner_secret (session, 3, "foo");
 
